﻿using Rhino;
using Rhino.Commands;
using Rhino.DocObjects;
using Rhino.Geometry;
using Rhino.Input.Custom;
using System.Collections.Generic;

namespace CustomObjectTest
{
    public class CustomObjectTestCommand : Command
    {
        public CustomObjectTestCommand()
        {
            // Rhino only creates one instance of each command class defined in a
            // plug-in, so it is safe to store a refence in a static property.
            Instance = this;
        }

        ///<summary>The only instance of this command.</summary>
        public static CustomObjectTestCommand Instance { get; private set; }

        ///<returns>The command name as it appears on the Rhino command line.</returns>
        public override string EnglishName => "CustomObjectTestCommand";

        protected override Result RunCommand(RhinoDoc doc, RunMode mode)
        {
            //------------------------------------------------
            // Test selection for ModelObject class development
            var gs = new GetObject();
            gs.SetCommandPrompt("Select surface");
            gs.GeometryFilter = ObjectType.Surface;
            //gs.DisablePreSelect();
            gs.SubObjectSelect = false;
            gs.GetMultiple(0, 0);
            if (gs.CommandResult() != Result.Success)
                return gs.CommandResult();

            List<Brep> brps = new List<Brep>();
            List<ObjRef> delete_refs = new List<ObjRef>();
            foreach (ObjRef r in gs.Objects())
            {
                Brep b = r.Brep();
                if (b.IsValid)
                {
                    brps.Add(b);
                    delete_refs.Add(r);
                }
            }

            // Convert to model objects
            List<TestCustomBrepObject> mos = new List<TestCustomBrepObject>();
            foreach (Brep b in brps)
            {
                TestCustomBrepObject mo = new TestCustomBrepObject(b);
                doc.Objects.AddRhinoObject(mo);
            }

            // Delete objects that have been converted, make optional later
            foreach (ObjRef r in delete_refs)
            {
                doc.Objects.Delete(r, true);
            }

            doc.Views.Redraw();
            return Result.Success;
        }
    }
}
