﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Drawing;
using Rhino.DocObjects.Custom;
using Rhino.Geometry;
using Rhino.DocObjects;
using Rhino.Display;
using Rhino;
using Rhino.Input.Custom;

namespace CustomObjectTest
{
    class TestCustomBrepObject : CustomBrepObject
    {
        // Example taken from: https://discourse.mcneel.com/t/rhinocommon-custom-objects/5403/6

        private BoundingBox _bb;

        /// <summary>
        /// 
        /// </summary>
        /// <param name="b"></param>
        public TestCustomBrepObject(Brep b)
            : base(b)
        {

        }

        /// <summary>
        /// 
        /// </summary>
        public TestCustomBrepObject()
        {
            // required
        }

        // TODO: Check which override is called when object is modfied via grips, by putting breakpoints in them one by one
        // See docs: https://developer.rhino3d.com/api/RhinoCommon/html/T_Rhino_DocObjects_Custom_CustomBrepObject.htm

        /// <summary>
        /// 
        /// </summary>
        /// <param name="doc"></param>
        protected override void OnAddToDocument(RhinoDoc doc)
        {
            // Not called when grips are modified.
            this._bb = this.Geometry.GetBoundingBox(true);
            base.OnAddToDocument(doc);
        }

        /// <summary>
        /// 
        /// </summary>
        /// <param name="doc"></param>
        protected override void OnDeleteFromDocument(RhinoDoc doc)
        {
            // called when grips are modified. But of no use?
            base.OnDeleteFromDocument(doc);
        }

        /// <summary>
        /// 
        /// </summary>
        /// <param name="e"></param>
        protected override void OnDraw(DrawEventArgs e)
        {
            // Called when viewport is moved
            if (this._bb.IsValid)
                e.Display.DrawBox(this._bb, Color.Red, 2);
            base.OnDraw(e);
        }

        /// <summary>
        /// 
        /// </summary>
        /// <param name="source"></param>
        protected override void OnDuplicate(RhinoObject source)
        {
            // Not called when grips are modified
            base.OnDuplicate(source);
        }

        /// <summary>
        /// 
        /// </summary>
        /// <param name="context"></param>
        /// <returns></returns>
        protected override IEnumerable<ObjRef> OnPick(PickContext context)
        {
            // TODO: Check when called
            return base.OnPick(context);
        }

        /// <summary>
        /// 
        /// </summary>
        /// <param name="context"></param>
        /// <param name="pickedItems"></param>
        protected override void OnPicked(PickContext context, IEnumerable<ObjRef> pickedItems)
        {
            // TODO: After object is picked. context has a sub objects field, but it is false for grips.
            base.OnPicked(context, pickedItems);
        }

        /// <summary>
        /// 
        /// </summary>
        protected override void OnSelectionChanged()
        {
            // TODO: Is called when grips are switched on. But cannot get useful grip info from instance.
            base.OnSelectionChanged();
        }

        /// <summary>
        /// 
        /// </summary>
        protected override void OnSwitchToNonConst()
        {
            // TODO: Is called when grips are switched on. Purpose unclear.
            base.OnSwitchToNonConst();
        }

        /// <summary>
        /// 
        /// </summary>
        /// <param name="transform"></param>
        protected override void OnTransform(Transform transform)
        {
            // Check: What type of transformations are captured.
            // - Edge crv. modifications with ctrl+shift selections: No, try to get sub object selection via OnPicked
            // - Gumbal transforms: Yes
            base.OnTransform(transform);
        }
    }
}
